//
//  GraphicLCDController.m
//  GraphicLCD
//
//  Created by Phidgets on 2018-02-15.
//  Copyright (c) 2018 Phidgets. All rights reserved.
//

#import "GraphicLCDController.h"
#define LCD_WIDTH_PIXELS 128
#define LCD_HEIGHT_PIXELS 64

@implementation GraphicLCDController

- (void)windowWillClose:(NSNotification*)aNotification {
    if(ch){
        //Safely close example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetLCD_delete(&ch);
        ch = NULL;
    }
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetLCD_create(&ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self addError:result];
    }

    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICEREMOTE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}

#pragma mark Event callbacks
static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}

static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}

void async_callback(PhidgetHandle phid, void *context, PhidgetReturnCode res){
    [(__bridge id)context performSelectorOnMainThread:@selector(asyncHandler:)
                                           withObject:[NSNumber numberWithInteger:res]
                                        waitUntilDone:NO];
}


#pragma mark Attach, detach, data, and error events
- (void)onAttachHandler{
    PhidgetReturnCode result;
    double minContrast, maxContrast, contrast, minBacklight, maxBacklight, backlight;
    
    //Get information from device
    result = PhidgetLCD_getMinContrast(ch, &minContrast);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_getMaxContrast(ch, &maxContrast);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_getContrast(ch, &contrast);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_getMinBacklight(ch, &minBacklight);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_getMaxBacklight(ch, &maxBacklight);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_getBacklight(ch, &backlight);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    //Adjust GUI based on information from channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [lcdBox setHidden:NO];
    
    [contrastSlider setMinValue:minContrast];
    [contrastSlider setMaxValue:maxContrast];
    [contrastSlider setDoubleValue:contrast];
    [minContrastLabel setStringValue:[NSString stringWithFormat:@"%.2f", minContrast]];
    [maxContrastLabel setStringValue:[NSString stringWithFormat:@"%.2f", maxContrast]];
    [contrastLabel setStringValue:[NSString stringWithFormat:@"%.2f", contrast]];
    
    [backlightSlider setMinValue:minBacklight];
    [backlightSlider setMaxValue:maxBacklight];
    [backlightSlider setDoubleValue:backlight];
    [minBacklightLabel setStringValue:[NSString stringWithFormat:@"%.2f", minBacklight]];
    [maxBacklightLabel setStringValue:[NSString stringWithFormat:@"%.2f", maxBacklight]];
    [backlightLabel setStringValue:[NSString stringWithFormat:@"%.2f", backlight]];
    
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow maxSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onDetachHandler{
    //Taking items out of view
    [lcdBox setHidden:YES];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

-(void)asyncHandler:(NSNumber *)result{
    if(result != EPHIDGET_OK){
        [self addError:(PhidgetReturnCode)result.intValue];
    }
    
}

#pragma mark GUI controls
- (IBAction)setContrast:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetLCD_setContrast(ch, [contrastSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [contrastLabel setStringValue:[NSString stringWithFormat:@"%0.3f",[contrastSlider doubleValue]]];
}

- (IBAction)setBacklight:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetLCD_setBacklight(ch, [backlightSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [backlightLabel setStringValue:[NSString stringWithFormat:@"%.3f ",[backlightSlider doubleValue]]];
}

- (IBAction)basicExample:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetLCD_setFrameBuffer(ch, 0);
    result = PhidgetLCD_clear(ch);
    
    uint8_t aBitmap[] = {
        0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00,
        0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00,
        0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00,
        0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00,
        0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00,
        0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00,
        0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00,
        0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00,
        0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00
    };
    
    uint8_t objcBitmap[] = {
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00,
        0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00,
        0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
    };
    
    
    result = PhidgetLCD_writeText(ch, FONT_6x12, 0, 0, "A few boldface A's");
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_setFontSize(ch, FONT_User1, 9, 10);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_setCharacterBitmap(ch, FONT_User1, "A", aBitmap);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_writeText(ch, FONT_User1, 0, 13, "AAAAAA");
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_drawLine(ch, 5, 40, 35, 30);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_drawRect(ch, 20, 35, 30, 45, YES, NO);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_copy(ch, 0, 0, 5, 30, 35, 45, 87, 30, NO);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_writeBitmap(ch, 35, 35, 47, 25, objcBitmap);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_drawPixel(ch, 64, 32, PIXEL_STATE_ON); //center of LCD screen
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_flush(ch);//write the frame buffer to the screen
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
}
- (IBAction)asyncExample:(id)sender {
    PhidgetReturnCode result;
    PhidgetLCD_setFrameBuffer(ch, 0);
    PhidgetLCD_clear(ch);
    
    //draw some stuff asynchronously
    for(int i = 0; i < 8; i++){
        PhidgetLCD_drawLine_async(ch, 0, i*8, (LCD_WIDTH_PIXELS -1), i*8, async_callback, (__bridge void*)self);
    }
    for(int i = 0; i < 21; i ++){
        PhidgetLCD_drawRect_async(ch, i*6, 0, ((i*6)+6), 7, YES, (i % 2) == 0 ? NO : YES, async_callback, (__bridge void*)self);
    }
    
    PhidgetLCD_copy_async(ch, 0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 2, 6, 2, 9, NO, async_callback, (__bridge void*)self);
    PhidgetLCD_copy_async(ch, 0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 4, 6, 4, 17, NO, async_callback, (__bridge void*)self);
    PhidgetLCD_copy_async(ch, 0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 2, 6, 2, 25, NO, async_callback, (__bridge void*)self);
    PhidgetLCD_copy_async(ch, 0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1), 6, 0, 33, NO, async_callback, (__bridge void*)self);
    PhidgetLCD_copy_async(ch, 0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 2, 6, 2, 41, NO, async_callback, (__bridge void*)self);
    PhidgetLCD_copy_async(ch, 0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 4, 6, 4, 49, NO, async_callback, (__bridge void*)self);
    PhidgetLCD_copy_async(ch, 0, 0, 0, 0, (LCD_WIDTH_PIXELS - 1) - 2, 6, 2, 57, NO, async_callback, (__bridge void*)self);
    result = PhidgetLCD_flush(ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
}

- (IBAction)asyncText:(id)sender {
    PhidgetReturnCode result;
    uint8_t funnyface[] = {
        0x01, 0x00, 0x00, 0x00, 0x00, 0x01,
        0x00, 0x01, 0x00, 0x00, 0x01, 0x00,
        0x00, 0x00, 0x01, 0x01, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x01, 0x00, 0x00, 0x01, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x01, 0x01, 0x00, 0x00,
        0x01, 0x00, 0x00, 0x00, 0x00, 0x01,
        0x01, 0x00, 0x00, 0x00, 0x00, 0x01,
        0x01, 0x01, 0x00, 0x00, 0x01, 0x01,
        0x00, 0x01, 0x01, 0x01, 0x01, 0x00
    };
    
    uint8_t space[] = {
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00
    };
    
    uint8_t thing1[] = {1,0,1,0,1,0,1,0,1,0};
    uint8_t thing2[] = {0,1,0,1,0,1,0,1,0,1};
    uint8_t space2[] = {0,0,0,0,0,0,0,0,0,0};
    
    result = PhidgetLCD_setFrameBuffer(ch, 0);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_clear(ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = PhidgetLCD_setFontSize(ch, FONT_User2, 6, 12);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_setCharacterBitmap(ch, FONT_User2, "0", funnyface);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_setCharacterBitmap(ch, FONT_User2, " ", space);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_writeText(ch, FONT_User2, 0, 35, "0 0 0");
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = PhidgetLCD_setFontSize(ch, FONT_User1, 1, 10);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_setCharacterBitmap(ch, FONT_User1, "1", thing1);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_setCharacterBitmap(ch, FONT_User1, "2", thing2);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_setCharacterBitmap(ch, FONT_User1, " ", space2);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    PhidgetLCD_writeText_async(ch, FONT_User1, 0, 0, "121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 12121212", nil, nil);
    PhidgetLCD_writeText_async(ch, FONT_User1, 0, 10, " 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 1212121", nil, nil);
    PhidgetLCD_writeText_async(ch, FONT_User1, 0, 20, "121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 121212121 12121212", nil, nil);
    
    PhidgetLCD_writeText_async(ch, FONT_6x12, 5, 5, "Overwriting!", nil, nil);
    result = PhidgetLCD_flush(ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
}

int runAsyncThread = 0;
- (IBAction)asyncTimer:(id)sender {
    runAsyncThread = !runAsyncThread;
    if(runAsyncThread){
        NSThread* backgroundThread = [[NSThread alloc] initWithTarget:self
                                                     selector:@selector(asyncTextThread)
                                                       object:nil];
        [backgroundThread start];
    }
}
- (IBAction)saveFrameBuffer:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetLCD_copy(ch, 0, 2, 0, 0, LCD_WIDTH_PIXELS-1, LCD_HEIGHT_PIXELS-1, 0, 0, NO);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_saveFrameBuffer(ch, 2);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
}
- (IBAction)loadFrameBuffer:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetLCD_setFrameBuffer(ch, 2);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_flush(ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
}
- (IBAction)sleep:(id)sender {
    NSButton *button = (NSButton *)sender;
    PhidgetReturnCode result;
    int sleeping = 0;
    
    result = PhidgetLCD_getSleeping(ch, &sleeping);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    if(sleeping){
        [button setTitle:@"Sleep"];
    }
    else{
        [button setTitle:@"Wake Up"];
    }
    
    result = PhidgetLCD_setSleeping(ch, sleeping ? 0 : 1);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
}
- (IBAction)clearScreen:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetLCD_setFrameBuffer(ch, 0);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_clear(ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    result = PhidgetLCD_flush(ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
}

#pragma mark Misc

-(void)asyncTextThread{
    while(runAsyncThread){
        int x = arc4random_uniform(127);
        int y = arc4random_uniform(63);
        PhidgetLCD_writeText_async(ch, FONT_5x8, x, y, [[NSString stringWithFormat:@"%d",arc4random_uniform(1000)] UTF8String], nil, nil);
        PhidgetLCD_flush_async(ch, nil, nil);
    }
}

#pragma mark Error handling
static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
}

-(void)addError:(PhidgetReturnCode)result{
    const char *errorString;
    Phidget_getErrorDescription(result, &errorString);
    [self outputError:errorString];
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}

@end
